'use strict';

var _rxjs = require('rxjs');

var _ssr = require('../ssr');

var _ssr2 = _interopRequireDefault(_ssr);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('ssr tests', function () {

  describe('mock tests', function () {

    it('snapshots the mock object', function () {
      expect(_ssr.SSR_DEPENDENCIES_MOCK).toMatchSnapshot();
    });

    it('tests that the "observe" mock function retruns the secend argument (the observable)', function () {
      var mockAction = 'mockAction';
      var mockObservable = 'mockObservable';
      expect(_ssr.SSR_DEPENDENCIES_MOCK.observe(mockAction, mockObservable)).toEqual(mockObservable);
    });
  });

  describe('SSR class tests', function () {

    var ssr = null;
    beforeEach(function () {
      var currentLocation = 'mock/location';
      ssr = new _ssr2.default(currentLocation);
    });

    it('snapshots the SSR.ACTION_TYPES', function () {
      expect(_ssr2.default.ACTION_TYPES).toMatchSnapshot();
    });

    it('tests that all class variables are set properly after initialization', function () {
      expect(ssr.pendingActions).toEqual([]);
      expect(ssr.store).toEqual(null);
      expect(ssr.initialAction).toMatchSnapshot();
      expect(ssr.loadingComplete).toBe(false);
      expect(ssr.actionCounter).toBe(0);
    });

    it('tests that "onLoad" sets the inner callback function and returns the class instance for chaining', function () {
      var onloadCallbackMock = 'onloadCallbackMock';
      var instance = ssr.onLoad(onloadCallbackMock);
      expect(ssr.onLoadCallback).toEqual(onloadCallbackMock);
      expect(instance).toBe(ssr);
    });

    it('tests that "onRedirect" sets the inner callback function and returns the class instance for chaining', function () {
      var onRedirectCallbackMock = 'onRedirectCallbackMock';
      var instance = ssr.onRedirect(onRedirectCallbackMock);
      expect(ssr.onRedirectCallback).toEqual(onRedirectCallbackMock);
      expect(instance).toBe(ssr);
    });

    it('tests that "onNotFound" sets the inner callback function and returns the class instance for chaining', function () {
      var onNotFoundCallbackMock = 'onNotFoundCallbackMock';
      var instance = ssr.onNotFound(onNotFoundCallbackMock);
      expect(ssr.onNotFoundCallback).toEqual(onNotFoundCallbackMock);
      expect(instance).toBe(ssr);
    });

    it('tests that "dispatchInitialAction" sets the store and dispatches the initial action', function () {
      var store = { dispatch: jest.fn() };
      ssr._setStore = jest.fn(function () {
        return ssr.store = store;
      });
      ssr._hasValidStore = jest.fn(function () {
        return true;
      });

      ssr.dispatchInitialAction(store);
      expect(ssr._setStore).toHaveBeenCalledWith(store);
      expect(ssr._hasValidStore).toHaveBeenCalled();
      expect(ssr.store.dispatch).toHaveBeenCalledWith(ssr.initialAction);
    });

    describe('middleware tests', function () {

      var middleware = null;
      var store = 'store';
      var next = jest.fn();
      var action = { type: 'SSR/MOCK_ACTION' };
      beforeEach(function () {
        ssr._onUpdate = jest.fn();
        next = jest.fn();

        middleware = ssr.middleware();
      });

      it('tests that the redux middleware reacts on actions starting with "SSR/" and updates pending actions in the stack', function () {
        middleware(store)(next)(action);
        expect(ssr._onUpdate).toHaveBeenCalledWith(action);
        expect(next).toHaveBeenCalledWith(action);
      });

      it('tests that the redux middleware doesn`t updates pending actions when loading is complete', function () {
        ssr.loadingComplete = true;
        middleware(store)(next)(action);
        expect(ssr._onUpdate).not.toHaveBeenCalled();
        expect(next).toHaveBeenCalledWith(action);
      });
    });

    it('test that "_onLoadingComplete" sets the inner loading flag to <true> and make the "epic functions" unusable', function () {
      _ssr.SSR_DEPENDENCIES_MOCK.notFound = jest.fn();
      _ssr.SSR_DEPENDENCIES_MOCK.redirect = jest.fn();
      _ssr.SSR_DEPENDENCIES_MOCK.observe = jest.fn();

      expect(ssr.loadingComplete).toBe(false);
      ssr._onLoadingComplete();
      expect(ssr.loadingComplete).toBe(true);

      ssr.notFound();
      expect(_ssr.SSR_DEPENDENCIES_MOCK.notFound).toHaveBeenCalled();

      ssr.redirect();
      expect(_ssr.SSR_DEPENDENCIES_MOCK.redirect).toHaveBeenCalled();

      ssr.observe();
      expect(_ssr.SSR_DEPENDENCIES_MOCK.observe).toHaveBeenCalled();
    });

    describe('observable method tests', function () {

      var action = void 0,
          observable = null;
      beforeEach(function () {
        action = { type: 'mockType' };
        observable = _rxjs.Observable.of(action);
        global.Date = jest.fn(function () {
          return {
            getTime: jest.fn(function () {
              return '123456789';
            })
          };
        });
      });

      it('tests that the "observe" method takes an action and a observable and adds an action object to pending actions', function () {
        ssr._addPendingAction = jest.fn();

        ssr.observe(action, observable);
        expect(ssr._addPendingAction).toHaveBeenCalledWith({
          actionType: action.type,
          time: '123456789'
        });
      });

      it('tests that the "observe" method zips the handed over observable and adds a custom observable to detect finish', function (done) {
        var store = { dispatch: jest.fn() };
        ssr._setStore(store);
        var zippedObservable = ssr.observe(action, observable);

        zippedObservable.subscribe(function (mappedAction) {
          if (mappedAction.time) expect(mappedAction).toEqual({
            type: _ssr2.default.ACTION_TYPES.SUCCESS,
            actionType: 'mockType',
            time: '123456789'
          });else expect(mappedAction).toEqual(action);
        }, function () {}, function () {
          done();
        });
      });
    });

    it('tests that the "redirect" method completes the loading state and calls the redirect callback with given url', function () {
      var store = 'mockStore';
      var redirectUrl = '/redirect/to/url';

      ssr._onLoadingComplete = jest.fn();
      ssr.onRedirectCallback = jest.fn();
      ssr._setStore(store);

      ssr.redirect(redirectUrl);
      expect(ssr._onLoadingComplete).toHaveBeenCalled();
      expect(ssr.onRedirectCallback).toHaveBeenCalledWith(store, {
        status: 301,
        redirectUrl: redirectUrl
      });
    });

    it('tests that the "notFound" method completes the loading state and calls the notFound callback with status 404', function () {
      var store = 'mockStore';

      ssr._onLoadingComplete = jest.fn();
      ssr.onNotFoundCallback = jest.fn();
      ssr._setStore(store);

      ssr.notFound();
      expect(ssr._onLoadingComplete).toHaveBeenCalled();
      expect(ssr.onNotFoundCallback).toHaveBeenCalledWith(store, { status: 404 });
    });

    it('tests that _setStore ... sets the store ... Oo', function () {
      var store = 'mockStore';
      ssr._setStore(store);
      expect(ssr.store).toEqual(store);
    });

    it('tests that the store is set', function () {
      ssr.store = 'mockStore';
      expect(ssr._hasValidStore()).toBe(true);
    });

    it('tests that _addPendingAction adds the action to the pending list and dispatches a <SSR/.*> action', function () {
      var store = { dispatch: jest.fn() };
      ssr._setStore(store);
      var action = {
        type: 'mockType',
        data: 'mockData'
      };
      ssr._addPendingAction(action);
      expect(ssr.pendingActions).toEqual([action]);
      expect(store.dispatch).toHaveBeenCalledWith({
        type: _ssr2.default.ACTION_TYPES.PENDING,
        data: 'mockData'
      });
    });

    describe('_onUpdate method', function () {

      it('tests that a ssr success action removes the corresponding pending action from the pending list', function () {
        var store = { dispatch: jest.fn() };
        ssr._setStore(store);
        var action = {
          type: 'mockType',
          time: '123456789'
        };

        ssr._onLoadingComplete = jest.fn();
        ssr.onLoadCallback = jest.fn();

        ssr._addPendingAction(action);
        ssr._onUpdate({
          type: _ssr2.default.ACTION_TYPES.SUCCESS,
          actionType: action.type,
          time: action.time
        });

        expect(ssr.pendingActions.length).toBe(0);
        expect(ssr._onLoadingComplete).toHaveBeenCalled();
        expect(ssr.onLoadCallback).toHaveBeenCalledWith(store);
      });
    });
  });
});